@@ -1,15 +1,21 @@
-#!/usr/bin/env python3
+r"""
+Custom scalar PDE
+=================
 
-from pde.grids import UnitGrid
-from pde.fields import ScalarField
-from pde.pdes.base import PDEBase
+This example implements a custom scalar PDE. To display some of the features of
+the package, we here implemented the `Kuramoto–Sivashinsky equation
+<https://en.wikipedia.org/wiki/Kuramoto–Sivashinsky_equation>`_, which was
+considered for flame front dynamics:
+
+.. math::
+    \partial_t u = -\frac12 |\nabla u|^2 - \nabla^2 u - \nabla^4 u
+"""
+
+from pde import UnitGrid, ScalarField, PDEBase
 
 
 class KuramotoSivashinskyPDE(PDEBase):
-    """ Kuramoto–Sivashinsky equation
-    
-    See https://en.wikipedia.org/wiki/Kuramoto–Sivashinsky_equation
-    """
+    """ Implementation of the normalized Kuramoto–Sivashinsky equation """
     
     def evolution_rate(self, state, t=0):
         """ implement the python version of the evolution equation """
@@ -19,9 +25,9 @@ class KuramotoSivashinskyPDE(PDEBase):
         return -state_grad.to_scalar('squared_sum') / 2 - state_lap - state_lap2 
 
 
-eq = KuramotoSivashinskyPDE()             # define the pde
 grid = UnitGrid([16, 16])                 # generate grid
 state = ScalarField.random_uniform(grid)  # generate initial condition
 
+eq = KuramotoSivashinskyPDE()             # define the pde
 result = eq.solve(state, t_range=10, dt=0.01)
 result.plot(show=True)
