@@ -2,7 +2,6 @@
 .. codeauthor:: David Zwicker <david.zwicker@ds.mpg.de>
 '''
 
-import itertools
 import tempfile
 
 import numpy as np
@@ -21,6 +20,7 @@ from ...tools.misc import skipUnlessModule
 def iter_grids():
     """ generator providing some test grids """
     for periodic in [True, False]:
+        yield UnitGrid([3], periodic=periodic)
         yield UnitGrid([3, 3, 3], periodic=periodic)
         yield CartesianGrid([[-1, 2], [0, 3]], [5, 7], periodic=periodic)
         yield CylindricalGrid(3, [-1, 2], [7, 8], periodic_z=periodic)
@@ -30,24 +30,25 @@ def iter_grids():
     
 
 @pytest.mark.parametrize("grid", iter_grids())
-@pytest.mark.parametrize("method", ["scipy_linear", "numba_linear"])
 @pytest.mark.parametrize("field_class", [ScalarField, Tensor2Field])
-def test_interpolation_natural(grid, method, field_class):
+def test_interpolation_natural(grid, field_class):
     """ test some interpolation for natural boundary conditions """
-    msg = f'grid={grid}, method={method}, field={field_class}'
+    msg = f'grid={grid}, field={field_class}'
     f = field_class.random_uniform(grid)
     if isinstance(grid, CartesianGridBase):
-        p = grid.get_random_point(boundary_distance=1)
+        p = grid.get_random_point(boundary_distance=.5)
     else:
         p = grid.get_random_point(boundary_distance=1, avoid_center=True)
     p = grid.point_from_cartesian(p)
     i1 = f.interpolate(p, method='scipy_linear')
-    i2 = f.interpolate(p, method='numba_linear')
+    i2 = f.interpolate(p, method='numba')
     np.testing.assert_almost_equal(i1, i2, err_msg=msg)
 
-    c = (2,) * len(grid.axes)  # specific cell
+    c = (1,) * len(grid.axes)  # specific cell
     p = f.grid.cell_coords[c]
-    np.testing.assert_allclose(f.interpolate(p, method=method),
+    np.testing.assert_allclose(f.interpolate(p, method='scipy_linear'),
+                               f.data[(Ellipsis,) + c], err_msg=msg)
+    np.testing.assert_allclose(f.interpolate(p, method='numba'),
                                f.data[(Ellipsis,) + c], err_msg=msg)
 
 
@@ -232,20 +233,16 @@ def test_writing_images():
            
 def test_interpolation_to_grid_fields():
     """ test whether data is interpolated correctly for different fields """
-    grid = UnitGrid([3, 4])
-    grid2 = UnitGrid([6, 8])
-    vf = VectorField.random_uniform(grid)
-    tf = Tensor2Field.random_uniform(grid)
-    sf = tf[0, 0]  # test extraction of fields
+    grid = CartesianGrid([[0, 2*np.pi]]*2, 6)
+    grid2 = CartesianGrid([[0, 2*np.pi]]*2, 8)
+    vf = VectorField.from_expression(grid, ['sin(y)', 'cos(x)'])
+    sf = vf[0]  # test extraction of fields
     fc = FieldCollection([sf, vf])
     
-    for normalized, f in itertools.product([True, False], [sf, vf, tf, fc]):
-        f2 = f.interpolate_to_grid(grid2, normalized=normalized,
-                                   method='nearest')
-        f3 = f2.interpolate_to_grid(grid, normalized=normalized,
-                                    method='nearest')
-        np.testing.assert_allclose(f.data, f3.data, rtol=1e-6,
-                                   err_msg='normalized=%s' % normalized)
+    for f in [sf, vf, fc]:
+        f2 = f.interpolate_to_grid(grid2, method='numba')
+        f3 = f2.interpolate_to_grid(grid, method='numba')
+        np.testing.assert_allclose(f.data, f3.data, atol=0.2, rtol=0.2)
             
       
            
@@ -255,21 +252,53 @@ def test_interpolation_values(field_cls):
     grid = UnitGrid([3, 4])
     f = field_cls.random_uniform(grid)
     
-    intp = f.make_interpolator()
+    intp = f.make_interpolator('numba')
     c = f.grid.cell_coords[2, 2]
     np.testing.assert_allclose(intp(c), f.data[..., 2, 2])
+    
+    with pytest.raises(ValueError):
+        intp(np.array([100, -100]))
+
+    res = f.make_interpolator('numba', fill=45)(np.array([100, -100]))
+    np.testing.assert_almost_equal(res, np.full(f.data_shape, 45))
 
 
 
-def test_interpolation_to_cartesian():
+@pytest.mark.parametrize('grid', [UnitGrid([6]),
+                                  PolarGrid(6, 4),
+                                  SphericalGrid(7, 4),
+                                  CylindricalGrid(6, (0, 8), (7, 8))])
+def test_interpolation_to_cartesian(grid):
     """ test whether data is interpolated correctly to Cartesian grid """
-    for grid in [CylindricalGrid(3, [-1, 2], [7, 8]),
-                 SphericalGrid(3, 4)]:
-        vf = VectorField.random_uniform(grid)
-        sf = vf[0]  # test extraction of fields
-        fc = FieldCollection([sf, vf])
-        grid_cart = UnitGrid([8, 8, 8])
-        fc.interpolate_to_grid(grid_cart)
+    dim = grid.dim
+    vf = VectorField(grid, 2)
+    sf = vf[0]  # test extraction of fields
+    fc = FieldCollection([sf, vf])
+    
+    # subset
+    grid_cart = UnitGrid([4] * dim)
+    for f in [sf, fc]:
+        res = f.interpolate_to_grid(grid_cart)
+        np.testing.assert_allclose(res.data, 2)
+    
+    # superset
+    grid_cart = UnitGrid([8] * dim)
+    for f in [sf, fc]:
+        res = f.interpolate_to_grid(grid_cart, fill=0)
+        assert res.data.min() == 0
+        assert res.data.max() == pytest.approx(2)
+        
+        
+        
+@pytest.mark.parametrize('grid', [PolarGrid(6, 4),
+                                  SphericalGrid(7, 4),
+                                  CylindricalGrid(6, (0, 8), (7, 8))])
+def test_get_cartesian_grid(grid):
+    """ test whether Cartesian grids can be created """
+    cart = grid.get_cartesian_grid(mode='valid')
+    assert cart.volume < grid.volume
+    cart = grid.get_cartesian_grid(mode='full')
+    assert cart.volume > grid.volume
         
 
         
@@ -285,8 +314,9 @@ def test_simple_plotting(grid):
     fc = FieldCollection([sf, vf])
     for f in [sf, vf, tf, fc]:
         f.plot()
-        f.plot('image')
         f.plot('line')
+        if grid.dim >= 2:
+            f.plot('image')
         if isinstance(f, VectorField) and grid.dim == 2:
             f.plot('vector')
         plt.close('all')
