@@ -11,6 +11,7 @@ from .test_generic import iter_grids
 from ..scalar import ScalarField
 from ..base import FieldBase
 from ...grids import UnitGrid, CartesianGrid, PolarGrid
+from ...grids.boundaries import DomainError
 from ...grids.tests.test_cartesian import _get_cartesian_grid
 from ...tools.misc import module_available, skipUnlessModule
 
@@ -24,14 +25,14 @@ def test_interpolation_singular():
     # test constant boundary conditions
     bc = [{'type': 'value', 'value': 1}, {'type': 'value', 'value': 5}]
     x = np.linspace(0, 1, 7).reshape((7, 1))
-    y = field.interpolate(x, method='numba_linear', bc=bc)
+    y = field.interpolate(x, method='numba', bc=bc)
     np.testing.assert_allclose(y, 1 + 4*x.ravel())
 
     # test derivative boundary conditions
     bc = [{'type': 'derivative', 'value': -2},
           {'type': 'derivative', 'value': 2}]
     x = np.linspace(0, 1, 7).reshape((7, 1))
-    y = field.interpolate(x, method='numba_linear', bc=bc)
+    y = field.interpolate(x, method='numba', bc=bc)
     np.testing.assert_allclose(y, 2 + 2*x.ravel())
 
 
@@ -71,6 +72,10 @@ def test_scalars():
     
     s2 = FieldBase.from_state(s1.state_serialized, grid=grid, data=s1.data)
     assert s1 == s2
+    
+    # test options for plotting images
+    if module_available("matplotlib"):
+        s1.plot(transpose=True, colorbar=True)
 
 
 
@@ -103,11 +108,8 @@ def test_gradient():
 def test_interpolation_to_grid(grid):
     """ test whether data is interpolated correctly for different grids """
     sf = ScalarField.random_uniform(grid)
-    for normalized in [True, False]:
-        sf2 = sf.interpolate_to_grid(grid, normalized=normalized,
-                                     method='nearest')
-        np.testing.assert_allclose(sf.data, sf2.data, rtol=1e-6,
-                                   err_msg='grid=%s' % grid)
+    sf2 = sf.interpolate_to_grid(grid, method='numba')
+    np.testing.assert_allclose(sf.data, sf2.data, rtol=1e-6)
 
 
         
@@ -190,7 +192,7 @@ def test_interpolation_inhomogeneous_bc():
     """ test field interpolation with inhomogeneous boundary condition """
     sf = ScalarField(UnitGrid([3, 3], periodic=False))
     x = 1 + np.random.random()
-    v = sf.interpolate([x, 0], method='numba_linear',
+    v = sf.interpolate([x, 0], method='numba',
                        bc=['natural', {'type': 'value', 'value': 'x'}])
     assert x == pytest.approx(v)
 
@@ -213,3 +215,100 @@ def test_to_scalar():
     sf = ScalarField.random_uniform(UnitGrid([3, 3]))
     np.testing.assert_allclose(sf.to_scalar().data, sf.data)    
     np.testing.assert_allclose(sf.to_scalar('squared_sum').data, sf.data**2)
+
+
+
+@pytest.mark.parametrize('grid', (g for g in iter_grids() if g.num_axes > 1))
+@pytest.mark.parametrize('method', ['integral', 'average'])
+def test_projection(grid, method):
+    """ test scalar projection """
+    sf = ScalarField.random_uniform(grid)
+    for ax in grid.axes:
+        sp = sf.project(ax, method=method)
+        assert sp.grid.dim < grid.dim
+        assert sp.grid.num_axes == grid.num_axes - 1
+        if method == 'integral':
+            assert sp.integral == pytest.approx(sf.integral)
+        elif method == 'average':
+            assert sp.average == pytest.approx(sf.average)
+            
+    with pytest.raises(ValueError):
+        sf.project('q')
+
+
+
+@pytest.mark.parametrize('grid', (g for g in iter_grids() if g.num_axes > 1))
+def test_slice(grid):
+    """ test scalar slicing """
+    sf = ScalarField(grid, 0.5)
+    p = grid.get_random_point()
+    for i in range(grid.num_axes):
+        sf_slc = sf.slice({grid.axes[i]: p[i]})
+        np.testing.assert_allclose(sf_slc.data, 0.5)
+        assert sf_slc.grid.dim < grid.dim
+        assert sf_slc.grid.num_axes == grid.num_axes - 1
+        
+    with pytest.raises(DomainError):
+        sf.slice({grid.axes[0]: -10})
+    with pytest.raises(ValueError):
+        sf.slice({'q': 0})
+
+
+
+def test_slice_positions():
+    """ test scalar slicing at standard positions """
+    grid = UnitGrid([3, 1])
+    sf = ScalarField(grid, np.arange(3).reshape(3, 1))
+    assert sf.slice({'x': 'min'}).data == 0
+    assert sf.slice({'x': 'mid'}).data == 1
+    assert sf.slice({'x': 'max'}).data == 2
+    with pytest.raises(ValueError):
+        sf.slice({'x': 'foo'})
+
+
+
+def test_poisson_solver_1d():
+    """ test the poisson solver on 1d grids """
+    # solve Laplace's equation
+    grid = UnitGrid([4])
+    field = ScalarField(grid)
+    res = field.solve_poisson([{'value': -1}, {'value': 3}])
+    np.testing.assert_allclose(res.data, grid.axes_coords[0] - 1)
+    
+    res = field.solve_poisson([{'value': -1}, {'derivative': 1}])
+    np.testing.assert_allclose(res.data, grid.axes_coords[0] - 1)
+
+    # test Poisson equation with 2nd Order BC
+    res = field.solve_poisson([{'value': -1}, 'extrapolate'])
+    
+    # solve Poisson's equation
+    grid = CartesianGrid([[0, 1]], 4)
+    field = ScalarField(grid, data=1)
+    
+    res = field.copy()
+    field.solve_poisson([{'value': 1}, {'derivative': 1}], out=res)
+    xs = grid.axes_coords[0]
+    np.testing.assert_allclose(res.data, 1 + 0.5 * xs**2, rtol=1e-2)
+    
+
+
+def test_poisson_solver_2d():
+    """ test the poisson solver on 2d grids """
+    grid = CartesianGrid([[0, 2 * np.pi]] * 2, 16)
+    bcs = [{'value': 'sin(y)'}, {'value': 'sin(x)'}]
+    
+    # solve Laplace's equation
+    field = ScalarField(grid)
+    res = field.solve_poisson(bcs)
+    xs = grid.cell_coords[..., 0]
+    ys = grid.cell_coords[..., 1]
+    
+    # analytical solution was obtained with Mathematica
+    expect = (np.cosh(np.pi - ys) * np.sin(xs) +
+              np.cosh(np.pi - xs) * np.sin(ys)) / np.cosh(np.pi)
+    np.testing.assert_allclose(res.data, expect, atol=1e-2, rtol=1e-2)
+    
+    # test more complex case for exceptions
+    res = field.solve_poisson([{'value': 'sin(y)'}, {'curvature': 'sin(x)'}])
+    
+    
\ No newline at end of file
