@@ -6,20 +6,22 @@ Bases classes
 '''
 
 import json
+import functools
 import logging
 from abc import ABCMeta, abstractmethod, abstractproperty
 from typing import (List, Tuple, Dict, Any, Union, Callable, Generator,
-                    TYPE_CHECKING)
+                    Sequence, TYPE_CHECKING)
 
 import numpy as np
 
 from ..tools.numba import jit
 from ..tools.cache import cached_property, cached_method
+from ..tools.docstrings import fill_in_docstring
 
 
 
 if TYPE_CHECKING:
-    from .boundaries.axes import Boundaries  # @UnusedImport
+    from .boundaries.axes import Boundaries, BoundariesData  # @UnusedImport
 
 
 
@@ -32,6 +34,11 @@ def _check_shape(shape) -> Tuple[int, ...]:
     """ checks the consistency of shape tuples """
     if not hasattr(shape, '__iter__'):
         shape = [shape]  # support single numbers
+        
+    if len(shape) == 0:
+        raise ValueError('Require at least one dimension')
+        
+    # convert the shape to a tuple of integers
     result = []
     for dim in shape:
         if dim == int(dim) and dim >= 1:
@@ -72,6 +79,23 @@ def discretize_interval(x_min: float, x_max: float, num: int) \
 
 
 
+class DomainError(ValueError):
+    """ exception indicating that point lies outside domain """ 
+    pass
+
+
+
+class DimensionError(ValueError):
+    """ exception indicating that dimensions were inconsistent """ 
+    pass
+
+
+
+class PeriodicityError(RuntimeError):
+    """ exception indicating that the grid periodicity is inconsistent """
+    pass
+
+
 
 class GridBase(metaclass=ABCMeta):
     """ Base class for all grids defining common methods and interfaces
@@ -107,7 +131,7 @@ class GridBase(metaclass=ABCMeta):
     axes: List[str]
     num_axes: int
     discretization: Any
-    cell_volume_data: float
+    cell_volume_data: Sequence[Union[float, np.ndarray]]
     axes_coords: Tuple
     axes_bounds: Tuple[Tuple[float, float], ...]
     periodic: List[bool]
@@ -115,7 +139,7 @@ class GridBase(metaclass=ABCMeta):
 
     def __init__(self):
         """ initialize the grid """
-        self._logger = logging.getLogger(self.__class__.__module__)
+        self._logger = logging.getLogger(self.__class__.__name__)
 
 
     def __init_subclass__(cls, **kwargs):  # @NoSelf
@@ -210,7 +234,7 @@ class GridBase(metaclass=ABCMeta):
             ValueError: if grids are not compatible
         """
         if not self.compatible_with(other):
-            raise ValueError('Grids incompatible')
+            raise ValueError(f'Grids {self} and {other} are incompatible')
         
         
     @property
@@ -228,7 +252,8 @@ class GridBase(metaclass=ABCMeta):
     @cached_property()
     def cell_volumes(self):
         """ :class:`numpy.ndarray`: volume of each cell """
-        return np.broadcast_to(self.cell_volume_data, self.shape)
+        vols = functools.reduce(np.outer, self.cell_volume_data)
+        return np.broadcast_to(vols, self.shape)
         
         
     def distance_real(self, p1, p2) -> float:
@@ -281,6 +306,12 @@ class GridBase(metaclass=ABCMeta):
     def get_random_point(self, boundary_distance: float = 0,
                          cartesian: bool = True): pass
              
+
+    def get_subgrid(self, indices: Sequence[int]) -> "GridBase":
+        """ return a subgrid of only the specified axes """
+        raise NotImplementedError('Subgrids are not implemented for class '
+                                  f'{self.__class__.__name__}')
+    
                          
     def plot(self):
         """ visualize the grid """
@@ -294,18 +325,49 @@ class GridBase(metaclass=ABCMeta):
         return np.mean(self.discretization)  # type: ignore
 
     
-    def integrate(self, data) -> float:
+    def integrate(self, data: np.ndarray,
+                  axes: Sequence[int] = None) -> np.ndarray:
         """ Integrates the discretized data over the grid
         
         Args:
-            data (array): The values at the support points of the grid that 
-                need to be integrated
+            data (:class:`numpy.ndarray`):
+                The values at the support points of the grid that need to be
+                integrated.
+            axes (list of int, optional):
+                The axes along which the integral is performed. If omitted, all
+                axes are integrated over.
         
         Returns:
             float: The values integrated over the entire grid
         """
-        data = np.broadcast_to(data, self.shape)
-        return float((data * self.cell_volume_data).sum())
+        # determine the volumes of the individual cells
+        if axes is None:
+            volume_list = self.cell_volume_data
+        else:
+            # use stored value for the default case of integrating over all axes
+            volume_list = [cell_vol if ax in axes else 1
+                           for ax, cell_vol in enumerate(self.cell_volume_data)]
+        cell_volumes = functools.reduce(np.outer, volume_list)
+        
+        # determine the axes over which we will integrate
+        if not isinstance(data, np.ndarray) or data.ndim < self.num_axes:
+            # deal with the case where data is not supplied for each support
+            # point, e.g., when a single scalar is integrated over the grid
+            data = np.broadcast_to(data, self.shape)
+            
+        elif data.ndim > self.num_axes:
+            # deal with the case where more than a single value is provided per
+            # support point, e.g., when a tensorial field is integrated
+            offset = data.ndim - self.num_axes
+            if axes is None:
+                # integrate over all axes of the grid
+                axes = tuple(range(offset, data.ndim))
+            else:
+                # shift the indices to account for the data shape
+                axes = tuple(offset + i for i in axes)
+        
+        # calculate integral using a weighted sum along the chosen axes
+        return (data * cell_volumes).sum(axis=axes)
     
     
     @cached_method()
@@ -334,45 +396,53 @@ class GridBase(metaclass=ABCMeta):
         """ return a compiled function returning the volume of a grid cell
         
         Args:
-            flat_index (bool): When True, cell_volumes are indexed by a single
-                integer into the flattened array.
+            flat_index (bool):
+                When True, cell_volumes are indexed by a single integer into the
+                flattened array.
                 
         Returns:
             function: returning the volume of the chosen cell
         """
-        if np.isscalar(self.cell_volume_data):
-            cell_volume_data = self.cell_volume_data
+        if all(np.isscalar(d) for d in self.cell_volume_data):
+            # all cells have the same volume
+            cell_volume = np.product(self.cell_volume_data)
+
             @jit
-            def cell_volume(*args) -> float:
-                return cell_volume_data
+            def get_cell_volume(*args) -> float:
+                return cell_volume  # type: ignore
+        
         else:
+            # some cells have a different volume
             cell_volumes = self.cell_volumes
             
             if flat_index:
                 @jit
-                def cell_volume(idx: int) -> float:
+                def get_cell_volume(idx: int) -> float:
                     return cell_volumes.flat[idx]  # type: ignore
             else:
                 @jit
-                def cell_volume(*args) -> float:
+                def get_cell_volume(*args) -> float:
                     return cell_volumes[args]  # type: ignore
             
-        return cell_volume  # type: ignore
+        return get_cell_volume  # type: ignore
     
     
-    def make_interpolator_compiled(self, method: str = 'linear', bc='natural') \
-            -> Callable:
-        """ return a compiled function for interpolating values on the grid
+    @fill_in_docstring
+    def make_interpolator_compiled(self, bc: "BoundariesData" = 'natural',
+                                   fill: float = None) -> Callable:
+        """ return a compiled function for linear interpolation on the grid
         
         This interpolator respects boundary conditions and can thus interpolate
         values in the whole grid volume. However, close to corners, the
         interpolation might not be optimal, in particular for periodic grids.
         
         Args:
-            method (str): Determines how the interpolation is done. Currently,
-                only linear interpolation is supported.
-            bc: Sets the boundary condition, which affects how values at the
-                boundary are determined
+            bc:
+                {ARG_BOUNDARIES}
+            fill (float, optional):
+                Determines how values out of bounds are handled. If `None`, a
+                `ValueError` is raised when out-of-bounds points are requested.
+                Otherwise, the given value is returned.
                 
         Returns:
             A function which returns interpolated values when called with
@@ -381,9 +451,6 @@ class GridBase(metaclass=ABCMeta):
             containing the field data and position is denotes the position in
             grid coordinates.
         """
-        if method != 'linear':
-            raise ValueError(f"Unsupported interpolation method: '{method}'")
-
         bcs = self.get_boundary_conditions(bc)
         
         if self.num_axes == 1:
@@ -392,16 +459,19 @@ class GridBase(metaclass=ABCMeta):
             dx = self.discretization[0]
             size = self.shape[0]
             normalize_point = self.make_normalize_point_compiled()
-            ev = bcs[0].get_point_evaluator()
+            ev = bcs[0].get_point_evaluator(fill=fill)
         
             @jit
-            def interpolate_single(data, point):
+            def interpolate_single(data: np.ndarray, point: np.ndarray) \
+                    -> np.ndarray:
                 """ obtain interpolated value of data at a point
                 
                 Args:
-                    data (:class:`numpy.ndarray`):  values at the grid points
-                    point (:class:`numpy.ndarray`): Coordinates of a single
-                        point in the grid coordinate system
+                    data (:class:`numpy.ndarray`):
+                        A 1d array of values at the grid points
+                    point (:class:`numpy.ndarray`):
+                        Coordinates of a single point in the grid coordinate
+                        system
                 
                 Returns:
                     :class:`numpy.ndarray`: The interpolated value at the point
@@ -409,7 +479,10 @@ class GridBase(metaclass=ABCMeta):
                 normalize_point(point)
                 c_l, d_l = divmod((point[0] - lo) / dx - 0.5, 1.)
                 if c_l < -1 or c_l > size - 1:
-                    raise ValueError('Point lies outside grid')
+                    if fill is None:
+                        raise DomainError('Point lies outside the grid')
+                    else:
+                        return fill
                 c_li = int(c_l)
                 c_hi = c_li + 1
                 return (1 - d_l) * ev(data, (c_li,)) + d_l * ev(data, (c_hi,))  
@@ -420,11 +493,12 @@ class GridBase(metaclass=ABCMeta):
             lo_x, lo_y = np.array(self.axes_bounds)[:, 0]
             dx, dy = self.discretization
             periodic_x, periodic_y = self.periodic
-            ev_x = bcs[0].get_point_evaluator()
-            ev_y = bcs[1].get_point_evaluator()
+            ev_x = bcs[0].get_point_evaluator(fill=fill)
+            ev_y = bcs[1].get_point_evaluator(fill=fill)
         
             @jit
-            def interpolate_single(data, point):
+            def interpolate_single(data: np.ndarray, point: np.ndarray) \
+                    -> np.ndarray:
                 """ obtain interpolated value of data at a point
                 
                 Args:
@@ -476,7 +550,10 @@ class GridBase(metaclass=ABCMeta):
                         # implement correctly.
         
                 if weight == 0:
-                    raise ValueError('Point lies outside grid')
+                    if fill is None:
+                        raise DomainError('Point lies outside the grid')
+                    else:
+                        return fill
                             
                 return value / weight
             
@@ -486,12 +563,13 @@ class GridBase(metaclass=ABCMeta):
             lo_x, lo_y, lo_z = np.array(self.axes_bounds)[:, 0]
             dx, dy, dz = self.discretization
             periodic_x, periodic_y, periodic_z = self.periodic
-            ev_x = bcs[0].get_point_evaluator()
-            ev_y = bcs[1].get_point_evaluator()
-            ev_z = bcs[2].get_point_evaluator()
+            ev_x = bcs[0].get_point_evaluator(fill=fill)
+            ev_y = bcs[1].get_point_evaluator(fill=fill)
+            ev_z = bcs[2].get_point_evaluator(fill=fill)
         
             @jit
-            def interpolate_single(data, point):
+            def interpolate_single(data: np.ndarray, point: np.ndarray) \
+                    -> np.ndarray:
                 """ obtain interpolated value of data at a point
                 
                 Args:
@@ -556,7 +634,10 @@ class GridBase(metaclass=ABCMeta):
                             # tedious to do correctly.
                                 
                 if weight == 0:
-                    raise ValueError('Point lies outside grid')
+                    if fill is None:
+                        raise DomainError('Point lies outside the grid')
+                    else:
+                        return fill
                             
                 return value / weight            
             
@@ -602,7 +683,7 @@ class GridBase(metaclass=ABCMeta):
                 """
                 c_l, d_l = divmod((point[0] - lo) / dx - 0.5, 1.)
                 if c_l < -1 or c_l > size - 1:
-                    raise ValueError('Point lies outside grid')
+                    raise DomainError('Point lies outside grid')
                 c_li = int(c_l)
                 c_hi = c_li + 1
                 
@@ -616,7 +697,7 @@ class GridBase(metaclass=ABCMeta):
 
                 elif c_li < 0:
                     if c_hi >= size:
-                        raise RuntimeError('Point lies outside grid')
+                        raise DomainError('Point lies outside the grid')
                     else:  # c_hi < size
                         data[..., c_hi] += amount / cell_volume(c_hi)
                 else:  # c_li >= 0
@@ -675,7 +756,7 @@ class GridBase(metaclass=ABCMeta):
                         total_weight += w_x[i] * w_y[j]
         
                 if total_weight == 0:
-                    raise ValueError('Point lies outside grid')
+                    raise DomainError('Point lies outside the grid')
         
                 # change the field with the correct weights
                 for i in range(2):
@@ -754,7 +835,7 @@ class GridBase(metaclass=ABCMeta):
                             total_weight += w_x[i] * w_y[j] * w_z[k]
         
                 if total_weight == 0:
-                    raise ValueError('Point lies outside grid')
+                    raise DomainError('Point lies outside the grid')
         
                 # change the field with the correct weights
                 for i in range(2):
@@ -787,5 +868,3 @@ class GridBase(metaclass=ABCMeta):
                                       f'for dimension {self.num_axes}')
             
         return add_interpolated  # type: ignore
-                
-
