@@ -11,11 +11,12 @@ non-periodic axes have more option, which are represented by
 
 from typing import Union, Tuple, Dict, Callable
     
-from ..base import GridBase
-from ...tools.numba import jit
+import numpy as np
+from numba.extending import register_jitable
+    
+from ..base import GridBase, DomainError
 
-from .local import (BCBase, BoundaryData, NeumannBC, _make_get_arr_1d,
-                    DomainError)
+from .local import BCBase, BoundaryData, NeumannBC, _make_get_arr_1d
 
 
 
@@ -235,6 +236,29 @@ class BoundaryPair(BoundaryAxisBase):
                 self.high.get_virtual_point_data())
 
 
+    def get_data(self, idx: Tuple[int, ...]) -> Tuple[float, Dict[int, float]]:
+        """ sets the elements of the sparse representation of this condition
+        
+        Args:
+            idx (tuple):
+                The index of the point that must lie on the boundary condition
+                
+        Returns:
+            float, dict: A constant value and a dictionary with indices and
+            factors that can be used to calculate this virtual point
+        """
+        axis_coord = idx[self.axis]
+        if axis_coord == -1:
+            # the virtual point on the lower side
+            return self.low.get_data(idx)
+        elif axis_coord == self.grid.shape[self.axis]:
+            # the virtual point on the upper side
+            return self.high.get_data(idx)
+        else:
+            # the normal case of an interior point
+            return 0, {axis_coord: 1}
+
+
     def get_virtual_point_evaluators(self) -> Tuple[Callable, Callable]:
         """ returns two functions evaluating the value at virtual support points
 
@@ -257,11 +281,17 @@ class BoundaryPair(BoundaryAxisBase):
         return self.__class__(self.low.differentiated, self.high.differentiated)
 
 
-    def get_point_evaluator(self) -> Callable:
+    def get_point_evaluator(self, fill: np.array = None) -> Callable:
         """ return a function to evaluate values at a given point
         
         The point can either be a point inside the domain or a virtual point
         right outside the domain
+
+        Args:
+            fill (:class:`numpy.ndarray`, optional):
+                Determines how values out of bounds are handled. If `None`, a
+                `DomainError` is raised when out-of-bounds points are requested.
+                Otherwise, the given value is returned.
             
         Returns:
             function: A function taking a 1d array and an index as an argument,
@@ -273,20 +303,31 @@ class BoundaryPair(BoundaryAxisBase):
         eval_lo = self.low.get_virtual_point_evaluator()
         eval_hi = self.high.get_virtual_point_evaluator()
 
-        @jit
+        @register_jitable
         def evaluate(arr, idx):
             """ evaluate values of the 1d array `arr_1d` at an index `i` """
             arr_1d, i, _ = get_arr_1d(arr, idx)
             
             if i == -1:
+                # virtual point on the lower side of the axis
                 return eval_lo(arr, idx)
+            
             elif i == size:
+                # virtual point on the upper side of the axis
                 return eval_hi(arr, idx)
+            
             elif 0 <= i < size:
+                # inner point of the axis
                 return arr_1d[..., i]
+            
+            elif fill is None:
+                # point is outside the domain and no fill value is specified
+                raise DomainError('Point index lies outside bounds')
+            
             else:
-                # point higher than right boundary
-                raise DomainError('Point index outside bounds')
+                # Point is outside the domain, but fill value is specified. Note
+                # that fill value needs to be given with the correct shape.
+                return fill
         
         return evaluate  # type: ignore
 
@@ -306,7 +347,7 @@ class BoundaryPair(BoundaryAxisBase):
         ap_low = self.low.get_adjacent_evaluator()
         ap_high = self.high.get_adjacent_evaluator()
         
-        @jit
+        @register_jitable
         def region_evaluator(arr, idx: Tuple[int, ...]) \
                 -> Tuple[float, float, float]:
             """ compiled function return the values in the region """
@@ -390,12 +431,12 @@ class BoundaryPeriodic(BoundaryAxisBase):
         """
         size = self.grid.shape[self.axis]
         
-        @jit
+        @register_jitable
         def value_low(arr):
             """ evaluate the virtual point using the data array `arr` """
             return arr[size - 1]
         
-        @jit
+        @register_jitable
         def value_high(arr):
             """ evaluate the virtual point using the data array `arr` """
             return arr[0]
@@ -412,7 +453,31 @@ class BoundaryPeriodic(BoundaryAxisBase):
         """     
         size = self.grid.shape[self.axis]
         return ((0., 1., size - 1, 0., 0),
-                (0., 1., 0, 0., 0))   
+                (0., 1., 0, 0., 0))
+
+
+    def get_data(self, idx: Tuple[int, ...]) -> Tuple[float, Dict[int, float]]:
+        """ sets the elements of the sparse representation of this condition
+        
+        Args:
+            idx (tuple):
+                The index of the point that must lie on the boundary condition
+                
+        Returns:
+            float, dict: A constant value and a dictionary with indices and
+            factors that can be used to calculate this virtual point
+        """
+        axis_coord = idx[self.axis]
+        size = self.grid.shape[self.axis]
+        if axis_coord == -1:
+            # the virtual point on the lower side
+            return 0, {size - 1: 1}
+        elif axis_coord == size:
+            # the virtual point on the upper side
+            return 0, {0: 1}
+        else:
+            # the normal case of an interior point
+            return 0, {axis_coord: 1}
 
 
     @property
@@ -421,11 +486,14 @@ class BoundaryPeriodic(BoundaryAxisBase):
         return self
 
 
-    def get_point_evaluator(self) -> Callable:
+    def get_point_evaluator(self, fill: float = None) -> Callable:
         """ return a function to evaluate values at a given point
         
         The point can either be a point inside the domain or a virtual point
-        right outside the domain
+        right outside the domain.
+        
+        Args:
+            fill: This argument is ignored.
         
         Returns:
             function: A function taking a 1d array and an index as an argument,
@@ -435,11 +503,11 @@ class BoundaryPeriodic(BoundaryAxisBase):
         
         get_arr_1d = _make_get_arr_1d(self.grid.num_axes, self.axis)
         
-        @jit
+        @register_jitable
         def evaluate(arr, idx):
             """ evaluate values of the array `arr` at an index `idx` """
             arr_1d, i, _ = get_arr_1d(arr, idx)
-            return arr_1d[i % size]  # wrap around for periodic boundaries
+            return arr_1d[..., i % size]  # wrap around for periodic boundaries
         
         return evaluate  # type: ignore
     
@@ -458,7 +526,7 @@ class BoundaryPeriodic(BoundaryAxisBase):
         size = self.grid.shape[self.axis]
         get_arr_1d = _make_get_arr_1d(self.grid.num_axes, self.axis)
 
-        @jit
+        @register_jitable
         def region_evaluator(arr, idx: Tuple[int, ...]) \
                 -> Tuple[float, float, float]:
             """ compiled function return the values in the region """
@@ -470,7 +538,7 @@ class BoundaryPeriodic(BoundaryAxisBase):
             ip = 0 if i == size - 1 else i + 1
            
             # return the values in the region around the point
-            return arr_1d[im], arr_1d[i], arr_1d[ip]
+            return arr_1d[..., im], arr_1d[..., i], arr_1d[..., ip]
         
         return region_evaluator  # type: ignore    
 
