@@ -32,9 +32,9 @@ import numbers
 from typing import Any, Union, Tuple, Dict, Sequence, Optional, Callable, List
     
 import numpy as np
+from numba.extending import register_jitable
 
 from ..base import GridBase
-from ...tools.numba import jit
 from ...tools.expressions import ScalarExpression
 
 
@@ -45,18 +45,6 @@ BoundaryData = Union[Dict, str, "BCBase"]
 
 
 
-class DomainError(ValueError):
-    """ exception indicating that point lies outside domain """ 
-    pass
-
-
-
-class PeriodicityError(RuntimeError):
-    """ exception indicating that the grid periodicity is inconsistent """
-    pass
-
-
-
 def _get_arr_1d(arr, idx: Tuple[int, ...], axis: int) \
         -> Tuple[Any, int, Tuple[int, ...]]:
     """ extract the 1d array along axis at point idx
@@ -130,7 +118,7 @@ def _make_get_arr_1d(dim: int, axis: int) -> Callable:
     """
     assert 0 <= axis < dim
     
-    @jit
+    @register_jitable
     def get_arr_1d(arr, idx: Tuple[int, ...]) \
             -> Tuple[Any, int, Tuple[int, ...]]:
         """ extract the 1d array along axis at point idx """
@@ -232,13 +220,22 @@ class BCBase(metaclass=ABCMeta):
             # parse the expression with the correct variables
             bc_vars = [self.grid.axes[i] for i in axes_ids]
             expr = ScalarExpression(value, bc_vars)
-            
-            # determine the value at each boundary position
+
+            # get the coordinates at each point of the boundary            
             bc_coords = np.meshgrid(*[self.grid.axes_coords[i]
                                       for i in axes_ids],
                                     indexing='ij')
-            self.value = expr(**{name: value
-                                 for name, value in zip(bc_vars, bc_coords)})
+            
+            # determine the value at each of these points. Note that we here
+            # iterate explicitly over all points because the expression might
+            # not depend on some of the variables, but we still want the array
+            # self.value to contain a value at each boundary point
+            self.value = np.empty_like(bc_coords[0])
+            coords = {name: 0 for name in bc_vars}
+            for idx in np.ndindex(*self.value.shape):
+                for i, name in enumerate(bc_vars):
+                    coords[name] = bc_coords[i][idx]
+                self.value[idx] = expr(**coords)
             
         elif np.isscalar(value):
             # homogeneous, scalar value
@@ -271,7 +268,7 @@ class BCBase(metaclass=ABCMeta):
                           for subclass in cls._subclasses.values()
                           if hasattr(subclass, 'names'))
         return (f"Possible types of boundary conditions are {types}. "
-                "Values can be set using {'type': TYPE, 'value': VALUE}."
+                "Values can be set using {'type': TYPE, 'value': VALUE}. "
                 "Here, VALUE can be a scalar number, a vector for tensorial "
                 "boundary conditions, or a string, which can be interpreted "
                 "as a sympy expression. In the latter case, the names of the "
@@ -282,12 +279,11 @@ class BCBase(metaclass=ABCMeta):
     def __repr__(self):
         if np.array_equal(self.value, 0):
             return (f"{self.__class__.__name__}("
-                    f"axis={self.axis}, upper={self.upper}, "
-                    f"homogeneous={self.homogeneous})")
+                    f"axis={self.axis}, upper={self.upper})")
         else:
             return (f"{self.__class__.__name__}("
                     f"axis={self.axis}, upper={self.upper}, "
-                    f"homogeneous={self.homogeneous}, value={self.value!r})")
+                    f"value={self.value!r})")
     
     
     def __str__(self):
@@ -534,6 +530,10 @@ class BCBase(metaclass=ABCMeta):
         return True
             
             
+    @abstractmethod
+    def get_data(self, idx: Tuple[int, ...]) -> Tuple[float, Dict[int, float]]:
+        pass
+            
     @abstractmethod
     def get_virtual_point(self, arr, idx: Tuple[int, ...] = None) -> float: pass
             
@@ -558,6 +558,32 @@ class BCBase1stOrder(BCBase):
     def get_virtual_point_data(self) -> Tuple[Any, Any, int]: pass
 
 
+    def get_data(self, idx: Tuple[int, ...]) -> Tuple[float, Dict[int, float]]:
+        """ sets the elements of the sparse representation of this condition
+        
+        Args:
+            idx (tuple):
+                The index of the point that must lie on the boundary condition
+                
+        Returns:
+            float, dict: A constant value and a dictionary with indices and
+            factors that can be used to calculate this virtual point
+        """
+        data = self.get_virtual_point_data()
+        
+        if self.homogeneous:
+            const = data[0]
+            factor = data[1]
+        else:
+            # obtain index of the boundary point
+            idx_c = list(idx)
+            del idx_c[self.axis]
+            const = data[0][tuple(idx_c)]
+            factor = data[1][tuple(idx_c)]
+            
+        return const, {data[2]: factor}
+
+
     def get_virtual_point(self, arr, idx: Tuple[int, ...] = None) -> float:
         """ calculate the value of the virtual point outside the boundary 
         
@@ -613,7 +639,7 @@ class BCBase1stOrder(BCBase):
         data = self.get_virtual_point_data()
         
         if self.homogeneous:
-            @jit
+            @register_jitable
             def virtual_point(arr, idx: Tuple[int, ...]) -> float:
                 """ evaluate the virtual point at `idx` """
                 arr_1d, _, _ = get_arr_1d(arr, idx)
@@ -621,7 +647,7 @@ class BCBase1stOrder(BCBase):
                         data[1] * arr_1d[..., data[2]])
                 
         else:
-            @jit
+            @register_jitable
             def virtual_point(arr, idx: Tuple[int, ...]) -> float:
                 """ evaluate the virtual point at `idx` """
                 arr_1d, _, bc_idx = get_arr_1d(arr, idx)
@@ -659,7 +685,7 @@ class BCBase1stOrder(BCBase):
             else:
                 zero = np.zeros((self.grid.dim,) * self.rank)
             
-            @jit
+            @register_jitable
             def adjacent_point(arr, idx: Tuple[int, ...]) -> float:
                 """ evaluate the point adjacent to `idx` """
                 # extract the 1d array
@@ -688,7 +714,7 @@ class BCBase1stOrder(BCBase):
         else:
             # the boundary condition is a function of space
             
-            @jit
+            @register_jitable
             def adjacent_point(arr, idx: Tuple[int, ...]) -> float:
                 """ evaluate the point adjacent to `idx` """
                 arr_1d, i, bc_idx = get_arr_1d(arr, idx)
@@ -791,6 +817,15 @@ class MixedBC(BCBase1stOrder):
     to Dirichlet conditions imposing :math:`\beta` as the derivative.
     Conversely,  :math:`\gamma \rightarrow \infty` corresponds to imposing a
     zero value on :math:`c`. 
+    
+    This condition can be enforced by using one of the following variants
+    
+    .. code-block:: python
+    
+        bc = {'mixed': VALUE}
+        bc = {'type': 'mixed', 'value': VALUE, 'const': CONST}
+        
+    where `VALUE` corresponds to :math:`\gamma` and `CONST` to :math:`\beta`.
     """
     
     names = ['mixed', 'robin']
@@ -887,6 +922,35 @@ class BCBase2ndOrder(BCBase):
             tuple: the data associated with this virtual point 
         """ 
 
+
+    def get_data(self, idx: Tuple[int, ...]) -> Tuple[float, Dict[int, float]]:
+        """ sets the elements of the sparse representation of this condition
+        
+        Args:
+            idx (tuple):
+                The index of the point that must lie on the boundary condition
+                
+        Returns:
+            float, dict: A constant value and a dictionary with indices and
+            factors that can be used to calculate this virtual point
+        """
+        data = self.get_virtual_point_data()
+        
+        if self.homogeneous:
+            const = data[0]
+            factor1 = data[1]
+            factor2 = data[3]
+        else:
+            # obtain index of the boundary point
+            idx_c = list(idx)
+            del idx_c[self.axis]
+            bc_idx = tuple(idx_c)
+            const = data[0][bc_idx]
+            factor1 = data[1][bc_idx]
+            factor2 = data[3][bc_idx]
+            
+        return const, {data[2]: factor1, data[4]: factor2}
+
     
     def get_virtual_point(self, arr, idx: Tuple[int, ...] = None) -> float:
         """ calculate the value of the virtual point outside the boundary 
@@ -949,7 +1013,7 @@ class BCBase2ndOrder(BCBase):
         data = self.get_virtual_point_data()
         
         if self.homogeneous:
-            @jit
+            @register_jitable
             def virtual_point(arr, idx: Tuple[int, ...]):
                 """ evaluate the virtual point at `idx` """
                 arr_1d, _, _ = get_arr_1d(arr, idx)
@@ -959,7 +1023,7 @@ class BCBase2ndOrder(BCBase):
                         data[3] * arr_1d[..., data[4]])
             
         else:
-            @jit
+            @register_jitable
             def virtual_point(arr, idx: Tuple[int, ...]):
                 """ evaluate the virtual point at `idx` """
                 arr_1d, _, bc_idx = get_arr_1d(arr, idx)
@@ -1002,7 +1066,7 @@ class BCBase2ndOrder(BCBase):
             else:
                 zero = np.zeros((self.grid.dim,) * self.rank)
             
-            @jit
+            @register_jitable
             def adjacent_point(arr, idx: Tuple[int, ...]):
                 """ evaluate the point adjacent to `idx` """
                 # extract the 1d array
@@ -1028,7 +1092,7 @@ class BCBase2ndOrder(BCBase):
         else:
             # the boundary condition is a function of space
             
-            @jit
+            @register_jitable
             def adjacent_point(arr, idx: Tuple[int, ...]):
                 """ evaluate the point adjacent to `idx` """
                 arr_1d, i, bc_idx = get_arr_1d(arr, idx)
