@@ -7,14 +7,16 @@ Cartesian grids of arbitrary dimension.
 
 import itertools
 from abc import ABCMeta
-from typing import List, Dict, Any, Union, Callable, Generator, TYPE_CHECKING
+from typing import (List, Sequence, Dict, Any, Union, Callable, Generator,
+                    TYPE_CHECKING)
 
 import numpy as np
 
 
-from .base import GridBase, _check_shape
+from .base import GridBase, _check_shape, DimensionError
 from ..tools.cache import cached_method
 from ..tools.cuboid import Cuboid
+from ..tools.docstrings import fill_in_docstring
 
 
 if TYPE_CHECKING:
@@ -25,7 +27,6 @@ if TYPE_CHECKING:
 class CartesianGridBase(GridBase, metaclass=ABCMeta):
     """Base class for :class:`UnitGrid` and :class:`CartesianGrid`"""
     
-    discretization: Any
     cuboid: Cuboid
     
     
@@ -48,9 +49,9 @@ class CartesianGridBase(GridBase, metaclass=ABCMeta):
         if isinstance(periodic, bool):
             self.periodic = [periodic] * self.dim
         elif len(periodic) != self.dim:
-            raise ValueError('Number of axes with specified periodicity does '
-                             f'not match grid dimension ({len(periodic)} != '
-                             f'{self.dim})')
+            raise DimensionError('Number of axes with specified periodicity '
+                                 'does not match grid dimension '
+                                 f'({len(periodic)} != {self.dim})')
         else:
             self.periodic = periodic
         
@@ -59,6 +60,12 @@ class CartesianGridBase(GridBase, metaclass=ABCMeta):
         else:
             self.axes = [chr(97 + i) for i in range(self.dim)]
         
+    
+    @property
+    def cell_volume_data(self):
+        """ size associated with each cell """
+        return tuple(self.discretization)
+        
 
     def contains_point(self, point):
         """ check whether the point is contained in the grid
@@ -67,7 +74,7 @@ class CartesianGridBase(GridBase, metaclass=ABCMeta):
             point (vector): Coordinates of the point
         """
         if len(point) != self.dim:
-            raise ValueError('Incompatible dimensions')
+            raise DimensionError('Incompatible dimensions')
         return self.cuboid.contains_point(point)
 
 
@@ -276,7 +283,7 @@ class CartesianGridBase(GridBase, metaclass=ABCMeta):
         """
         origin = np.array(origin, dtype=np.double, ndmin=1)
         if len(origin) != self.dim:
-            raise ValueError('Dimensions are not compatible')
+            raise DimensionError('Dimensions are not compatible')
         
         # calculate the difference vector between all cells and the origin
         diff = self.difference_vector_real(origin, self.cell_coords)
@@ -371,13 +378,13 @@ class CartesianGridBase(GridBase, metaclass=ABCMeta):
                                       f'of dimension {self.dim}')
             
 
+    @fill_in_docstring
     def get_boundary_conditions(self, bc='natural') -> "Boundaries":
         """ constructs boundary conditions from a flexible data format
         
         Args:
             bc (str or list or tuple or dict):
-                Boundary conditions specified in the flexible format accepted by
-                :func:`pde.grids.boundaries.Boundaries.from_data`.  
+                {ARG_BOUNDARIES}  
 
         Raises:
             ValueError: If the data given in `bc` cannot be read
@@ -389,6 +396,7 @@ class CartesianGridBase(GridBase, metaclass=ABCMeta):
     
     
     @cached_method()
+    @fill_in_docstring
     def get_operator(self, op: str, bc, method: str = 'auto') -> Callable:
         """ return a discretized operator defined on this grid
         
@@ -396,8 +404,7 @@ class CartesianGridBase(GridBase, metaclass=ABCMeta):
             op (str): Identifier for the operator. Some examples are 'laplace',
                 'gradient', or 'divergence'.
             bc (str or list or tuple or dict):
-                Boundary conditions specified in the flexible format accepted by
-                :func:`pde.grids.boundaries.Boundaries.from_data`.  
+                {ARG_BOUNDARIES}  
             method (str): Specifies the method used for creating the operator.
                 Typical values could be 'numba' and 'scipy'.
                 
@@ -426,9 +433,6 @@ class UnitGrid(CartesianGridBase):
     """
     
     
-    cell_volume_data = 1  # factor to convert cell volume to real volume
-    
-    
     def __init__(self, shape, periodic: Union[List[bool], bool] = False):
         """ 
         Args:
@@ -494,12 +498,12 @@ class UnitGrid(CartesianGridBase):
             return np.zeros((0, self.dim))
         if point.ndim == 0:
             if self.dim > 1:
-                raise ValueError(f'Dimension mismatch: Point {point} is '
-                                 f'not of dimension {self.dim}.')
+                raise DimensionError(f'Dimension mismatch: Point {point} is '
+                                     f'not of dimension {self.dim}.')
         elif point.shape[-1] != self.dim:
-            raise ValueError('Dimension mismatch: Array of shape '
-                             f'{point.shape} does not describe points of '
-                             f'dimension {self.dim}.')
+            raise DimensionError('Dimension mismatch: Array of shape '
+                                 f'{point.shape} does not describe points of '
+                                 f'dimension {self.dim}.')
         
         # normalize the coordinates for the periodic dimensions
         if point.ndim == 0:
@@ -530,7 +534,7 @@ class UnitGrid(CartesianGridBase):
         if cells.size == 0:
             return np.zeros((0, self.dim))
         if cells.shape[-1] != self.dim:
-            raise ValueError('Dimension mismatch')
+            raise DimensionError('Dimension mismatch')
         return cells + 0.5
 
 
@@ -576,6 +580,12 @@ class UnitGrid(CartesianGridBase):
         """ convert unit grid to CartesianGrid """
         return CartesianGrid(self.cuboid.bounds, shape=self.shape,
                              periodic=self.periodic)
+        
+        
+    def get_subgrid(self, indices: Sequence[int]) -> "UnitGrid":
+        """ return a subgrid of only the specified axes """
+        return self.__class__(shape=[self.shape[i] for i in indices],
+                              periodic=[self.periodic[i] for i in indices])
 
 
 
@@ -645,8 +655,8 @@ class CartesianGrid(CartesianGridBase):
         if len(shape) == 1 and self.cuboid.dim > 1:
             shape = np.full(self.cuboid.dim, shape, dtype=np.uint32)
         if self.cuboid.dim != len(shape):
-            raise ValueError('Dimension of the bounds and the shape are not '
-                             'compatible')
+            raise DimensionError('Dimension of the bounds and the shape are '
+                                 'not compatible')
                     
         # initialize the base class            
         super().__init__(shape, periodic)
@@ -667,11 +677,11 @@ class CartesianGrid(CartesianGridBase):
         self.axes_coords = tuple(axes_coords)
         self.axes_bounds = tuple(self.cuboid.bounds)
 
-
+    
     @property
     def state(self) -> Dict[str, Any]:
         """ dict: the state of the grid """
-        return {'bounds': tuple(tuple(cs) for cs in self.cuboid.bounds),
+        return {'bounds': self.axes_bounds,
                 'shape': self.shape,
                 'periodic': self.periodic}
 
@@ -700,12 +710,6 @@ class CartesianGrid(CartesianGridBase):
         return float(self.cuboid.volume)
 
 
-    @property    
-    def cell_volume_data(self) -> float:  # type: ignore
-        """ float: volume of a single cell """
-        return np.prod(self.discretization)  # type: ignore
-
-
     def normalize_point(self, point, reduced_coords: bool = False):
         """ normalize coordinates by applying periodic boundary conditions
         
@@ -723,12 +727,12 @@ class CartesianGrid(CartesianGridBase):
             return np.zeros((0, self.dim)) 
         if point.ndim == 0:
             if self.dim > 1:
-                raise ValueError(f'Dimension mismatch: Point {point} is '
-                                 f'not of dimension {self.dim}.')
+                raise DimensionError(f'Dimension mismatch: Point {point} is '
+                                     f'not of dimension {self.dim}.')
         elif point.shape[-1] != self.dim:
-            raise ValueError('Dimension mismatch: Array of shape '
-                             f'{point.shape} does not describe points of '
-                             f'dimension {self.dim}.')
+            raise DimensionError('Dimension mismatch: Array of shape '
+                                 f'{point.shape} does not describe points of '
+                                 f'dimension {self.dim}.')
         
         if any(self.periodic):
             if self.dim == 1:  # single, periodic dimension
@@ -802,3 +806,9 @@ class CartesianGrid(CartesianGridBase):
                             (diff[..., self.periodic] + size/2) % size - size/2
         return diff
 
+
+    def get_subgrid(self, indices: Sequence[int]) -> "CartesianGrid":
+        """ return a subgrid of only the specified axes """
+        return self.__class__(bounds=[self.axes_bounds[i] for i in indices],
+                              shape=[self.shape[i] for i in indices],
+                              periodic=[self.periodic[i] for i in indices])
