@@ -5,19 +5,20 @@ Cylindrical grids with azimuthal symmetry
  
 '''
 
-from typing import Tuple, Dict, Any, Union, Callable, Generator, TYPE_CHECKING
+from typing import (Tuple, Sequence, Dict, Union, Any, Callable, Generator,
+                    TYPE_CHECKING)
 
 import numpy as np
-from scipy import interpolate
 
-from .base import GridBase, discretize_interval, _check_shape
-from .cartesian import CartesianGridBase, CartesianGrid
+from .base import GridBase, discretize_interval, _check_shape, DimensionError
+from .cartesian import CartesianGrid
 from ..tools.cache import cached_property, cached_method
+from ..tools.docstrings import fill_in_docstring
 
 
 if TYPE_CHECKING:
     from .boundaries import Boundaries  # @UnusedImport
-
+    from .spherical import PolarGrid  # @UnusedImport
 
 
     
@@ -28,7 +29,6 @@ class CylindricalGrid(GridBase):
     coordinates :math:`r` and :math:`z`, respectively. These are discretized
     uniformly as
     
-    
     .. math::
         :nowrap:
     
@@ -78,7 +78,7 @@ class CylindricalGrid(GridBase):
         elif len(shape_list) == 2:
             self.shape = shape_list  # type: ignore
         else:
-            raise ValueError("`shape` must be two integers")
+            raise DimensionError("`shape` must be two integers")
         if len(bounds_z) != 2:
             raise ValueError('Lower and upper value of the axial coordinate '
                              'must be specified')
@@ -177,7 +177,6 @@ class CylindricalGrid(GridBase):
                 raise RuntimeError('Random points would be too close to '
                                    'boundary')
 
-
         # create random point
         r = r_mag * np.random.random() + r_min
         z = z_min + (z_max - z_min) * np.random.random()
@@ -295,12 +294,13 @@ class CylindricalGrid(GridBase):
 
 
     @cached_property()
-    def cell_volume_data(self):
+    def cell_volume_data(self) -> Tuple[np.ndarray, float]:
         """ :class:`numpy.ndarray`: the volumes of all cells """
         dr, dz = self.discretization
         rs = np.arange(self.shape[0] + 1) * dr
         areas = np.pi * rs**2
-        return (np.diff(areas) * dz).reshape(self.shape[0], 1)
+        r_vols = np.diff(areas).reshape(self.shape[0], 1)
+        return (r_vols, dz)
     
             
     def normalize_point(self, point, reduced_coords: bool = False):
@@ -320,9 +320,9 @@ class CylindricalGrid(GridBase):
         if point.size == 0:
             return np.zeros((0, size))
         if point.shape[-1] != size:
-            raise ValueError('Dimension mismatch: Array of shape '
-                             f'{point.shape} does not describe points of '
-                             f'dimension {size}.')
+            raise DimensionError('Dimension mismatch: Array of shape '
+                                 f'{point.shape} does not describe points of '
+                                 f'dimension {size}.')
         
         if self.periodic_z:
             z_min = self.axes_bounds[1][0]
@@ -344,7 +344,7 @@ class CylindricalGrid(GridBase):
         """
         points = np.atleast_1d(points)
         if points.shape[-1] != 2:
-            raise ValueError(f'Dimension mismatch: Points {points} invalid')
+            raise DimensionError(f'Dimension mismatch: Points {points} invalid')
         
         x = points[..., 0]
         y = np.zeros_like(x)
@@ -393,7 +393,7 @@ class CylindricalGrid(GridBase):
         if cells.size == 0:
             return np.zeros((0, self.dim))
         if cells.shape[-1] != 2:
-            raise ValueError(f'Dimension mismatch: Cell {cells} invalid')
+            raise DimensionError(f'Dimension mismatch: Cell {cells} invalid')
 
         # convert from cells indices to grid coordinates
         points = (cells + 0.5) * self.discretization
@@ -458,7 +458,7 @@ class CylindricalGrid(GridBase):
         """
         origin = np.array(origin, dtype=np.double, ndmin=1)
         if len(origin) != self.dim:
-            raise ValueError('Dimensions are not compatible')
+            raise DimensionError('Dimensions are not compatible')
             
         if origin[0] != 0 or origin[1] != 0:
             raise RuntimeError('Origin must lie on symmetry axis for '
@@ -474,13 +474,13 @@ class CylindricalGrid(GridBase):
             return dist
 
 
+    @fill_in_docstring
     def get_boundary_conditions(self, bc='natural') -> "Boundaries":
         """ constructs boundary conditions from a flexible data format
         
         Args:
             bc (str or list or tuple or dict):
-                Boundary conditions specified in the flexible format accepted by
-                :func:`pde.grids.boundaries.Boundaries.from_data`.  
+                {ARG_BOUNDARIES}  
 
         Raises:
             ValueError: If the data given in `bc` cannot be read
@@ -492,6 +492,7 @@ class CylindricalGrid(GridBase):
     
     
     @cached_method()
+    @fill_in_docstring
     def get_operator(self, op: str, bc) -> Callable:
         """ return a discretized operator defined on this grid
         
@@ -499,8 +500,7 @@ class CylindricalGrid(GridBase):
             op (str): Identifier for the operator. Some examples are 'laplace',
                 'gradient', or 'divergence'.
             bc (str or list or tuple or dict):
-                Boundary conditions specified in the flexible format accepted by
-                :func:`pde.grids.boundaries.Boundaries.from_data`.  
+                {ARG_BOUNDARIES}  
                 
         Returns:
             A function that takes the discretized data as an input and returns
@@ -513,62 +513,53 @@ class CylindricalGrid(GridBase):
                                          bcs=self.get_boundary_conditions(bc))
 
     
-    def interpolate_to_cartesian(self, data,
-                                 grid: Union[CartesianGridBase, str] = 'valid',
-                                 ret_grid: bool = False):
-        """ return the given cylindrical `data` in a 3d Cartesian grid
+    def get_cartesian_grid(self, mode: str = 'valid') -> CartesianGrid:
+        """ return a Cartesian grid for this Cylindrical one 
         
         Args:
-            data (:class:`numpy.ndarray`):
-                The actual data values on the cylindrical grid.
-            grid (str or CartesianGridBase):
-                determines the grid on which the data is returned. This can
-                either be an instance of CartesianGridBase, the string 'valid'
-                or the string 'full'. Using 'valid' only returns points that are
-                fully resolved in the cylindrical grid using an inscribed
-                cylinder. Conversely, 'full' returns all data, using a
-                circumscribed cylinder. 
-            ret_grid (bool):
-                Determines whether the respective Rectangular grid should be
-                returned alongside the data.
+            mode (str):
+                Determines how the grid is determined. Setting it to 'valid'
+                only returns points that are fully resolved in the cylindrical
+                grid, e.g., the cylinder is circumscribed. Conversely, 'full'
+                returns all data, so the cylinder is inscribed. 
                 
         Returns:
-            :class:`numpy.ndarray`: The data values on the Cartesian grid.
-            If `ret_grid=True`, a tuple of these data values and the associated
-            grid are returned. 
+            :class:`pde.grids.cartesian.CartesianGrid`: The requested grid
         """
-        assert data.shape == self.shape
-        
-        if isinstance(grid, GridBase):
-            # supplied grid is already constructed 
-            if not isinstance(grid, CartesianGridBase):
-                raise TypeError('Supplied grid needs to be of type '
-                                '`CartesianGridBase`')
+        # Pick the grid instance
+        if mode == 'valid':
+            bounds = self.radius / np.sqrt(self.dim)
+        elif mode == 'full':
+            bounds = self.radius
         else:
-            # need to build the grid instance
-            if grid == 'valid':
-                bounds = self.radius / np.sqrt(2)
-            elif grid == 'full':
-                bounds = self.radius
-            else:
-                raise ValueError('Do not support grid `{grid:s}`')
+            raise ValueError(f'Unsupported mode `{mode}`')
                 
-            num = round(bounds / self.discretization[0])
-            grid_bounds = [(-bounds, bounds), (-bounds, bounds),
-                           self.axes_bounds[1]]
-            grid_shape = 2*num, 2*num, self.shape[1]
-            grid = CartesianGrid(grid_bounds, grid_shape)
-        
-        xs, ys, zs = np.meshgrid(*grid.axes_coords, indexing='ij')
-        rs = np.hypot(xs, ys)
-        
-        # interpolate over the new coordinates
-        f = interpolate.Rbf(*np.meshgrid(*self.axes_coords, indexing='ij'),
-                            data, function='cubic')
-        data_int = f(rs, zs)
+        # determine the Cartesian grid
+        num = round(bounds / self.discretization[0])
+        grid_bounds = [(-bounds, bounds), (-bounds, bounds),
+                       self.axes_bounds[1]]
+        grid_shape = 2 * num, 2 * num, self.shape[1]
+        return CartesianGrid(grid_bounds, grid_shape)
+    
 
-        if ret_grid:
-            return data_int, grid
+    def get_subgrid(self, indices: Sequence[int]) \
+            -> Union["CartesianGrid", "PolarGrid"]:
+        """ return a subgrid of only the specified axes """
+        if len(indices) != 1:
+            raise ValueError(f'Can only get sub-grid for one axis.')
+        
+        if indices[0] == 0:
+            # return a radial grid
+            assert self.axes[0] == 'r'
+            from .spherical import PolarGrid  # @Reimport
+            return PolarGrid(self.radius, self.shape[0])
+            
+        elif indices[0] == 1:
+            # return a Cartesian grid along the z-axis
+            assert self.axes[1] == 'z'
+            return CartesianGrid(bounds=[self.axes_bounds[1]],
+                                 shape=self.shape[1],
+                                 periodic=self.periodic[1])
+            
         else:
-            return data_int
-    
\ No newline at end of file
+            raise ValueError(f'Cannot get sub-grid for index {indices[0]}')
