@@ -12,10 +12,11 @@ from typing import Tuple, Dict, Any, Union, Callable, Generator, TYPE_CHECKING
 import numpy as np
 from scipy import interpolate
 
-from .base import GridBase, discretize_interval, _check_shape
-from .cartesian import CartesianGridBase, CartesianGrid
+from .base import GridBase, discretize_interval, _check_shape, DimensionError
+from .cartesian import CartesianGrid
 from ..tools.spherical import volume_from_radius
 from ..tools.cache import cached_property, cached_method
+from ..tools.docstrings import fill_in_docstring
 
 
 
@@ -37,13 +38,15 @@ class SphericalGridBase(GridBase, metaclass=ABCMeta):
     
     
     .. math::
-        r_i = r_\mathrm{inner} + \left(i + \frac12\right) \Delta r
+        r_i = R_\mathrm{inner} + \left(i + \frac12\right) \Delta r
         \quad \text{for} \quad i = 0, \ldots, N - 1
-        \quad \text{with} \quad \Delta r = \frac{R}{N}
+        \quad \text{with} \quad 
+            \Delta r = \frac{R_\mathrm{outer} - R_\mathrm{inner}}{N}
 
-    where :math:`R` is the radius of the grid and :math:`r_\mathrm{inner}` 
-    corresponds to a possible inner radius, which is zero by default. The radial
-    direction is discretized by :math:`N` support points.    
+    where :math:`R_\mathrm{outer}` is the outer radius of the grid and
+    :math:`R_\mathrm{inner}` corresponds to a possible inner radius, which is
+    zero by default. The radial direction is discretized by :math:`N` support
+    points.    
     """
 
     periodic = [False]  # the radial axis is not periodic
@@ -52,11 +55,12 @@ class SphericalGridBase(GridBase, metaclass=ABCMeta):
 
     def __init__(self, radius: Union[float, Tuple[float, float]],
                  shape: Union[Tuple[int], int]):
-        """ 
+        r""" 
         Args:
             radius (float or tuple of floats): 
-                radius :math:`R` in case a simple float is given. If a tuple is
-                supplied it is interpreted as the inner and outer radius.
+                radius :math:`R_\mathrm{outer}` in case a simple float is given.
+                If a tuple is supplied it is interpreted as the inner and outer
+                radius, :math:`(R_\mathrm{inner}, R_\mathrm{outer})`.
             shape (tuple or int): A single number setting the number :math:`N`
                 of support points along the radial coordinate
         """
@@ -135,13 +139,13 @@ class SphericalGridBase(GridBase, metaclass=ABCMeta):
 
 
     @cached_property()
-    def cell_volume_data(self):
-        """ :class:`numpy.ndarray`: the volumes of all cells """
+    def cell_volume_data(self) -> Tuple[np.ndarray]:
+        """ tuple of :class:`numpy.ndarray`: the volumes of all cells """
         dr = self.discretization[0]
         rs = self.axes_coords[0]
         volumes_h = volume_from_radius(rs + 0.5 * dr, dim=self.dim)
         volumes_l = volume_from_radius(rs - 0.5 * dr, dim=self.dim)
-        return (volumes_h - volumes_l).reshape(self.shape[0])
+        return ((volumes_h - volumes_l).reshape(self.shape[0]),)  # type: ignore
     
 
     def contains_point(self, point):
@@ -152,7 +156,7 @@ class SphericalGridBase(GridBase, metaclass=ABCMeta):
         """
         point = np.atleast_1d(point)
         if point.shape[-1] != self.dim:
-            raise ValueError('Dimension mismatch')
+            raise DimensionError('Dimension mismatch')
         r = np.linalg.norm(point, axis=-1)
         
         r_inner, r_outer = self.axes_bounds[0]
@@ -303,9 +307,9 @@ class SphericalGridBase(GridBase, metaclass=ABCMeta):
         if point.size == 0:
             return np.zeros((0, size))
         if point.shape[-1] != size:
-            raise ValueError('Dimension mismatch: Array of shape '
-                             f'{point.shape} does not describe points of '
-                             f'dimension {size}.')
+            raise DimensionError('Dimension mismatch: Array of shape '
+                                 f'{point.shape} does not describe points of '
+                                 f'dimension {size}.')
         return point
 
     
@@ -412,6 +416,7 @@ class SphericalGridBase(GridBase, metaclass=ABCMeta):
             return rs
 
 
+    @fill_in_docstring
     def get_boundary_conditions(self, bc='natural') -> "Boundaries":
         """ constructs boundary conditions from a flexible data format.
         
@@ -420,8 +425,7 @@ class SphericalGridBase(GridBase, metaclass=ABCMeta):
         
         Args:
             bc (str or list or tuple or dict):
-                Boundary conditions specified in the flexible format accepted by
-                :func:`pde.grids.boundaries.Boundaries.from_data`.  
+                {ARG_BOUNDARIES}
 
         Raises:
             ValueError: If the data given in `bc` cannot be read
@@ -432,6 +436,9 @@ class SphericalGridBase(GridBase, metaclass=ABCMeta):
         from .boundaries.axis import BoundaryPair
         from .boundaries import Boundaries  # @Reimport
 
+        if isinstance(bc, Boundaries):
+            return bc
+
         if self.has_hole:  # grid has holes => specify two boundary conditions
             if bc == 'natural':
                 b_pair = BoundaryPair(NeumannBC(self, 0, upper=False),
@@ -462,68 +469,38 @@ class SphericalGridBase(GridBase, metaclass=ABCMeta):
         return Boundaries([b_pair])       
 
     
-    def interpolate_to_cartesian(self, data,
-                                 grid: Union[CartesianGridBase, str] = 'valid',
-                                 ret_grid: bool = False):
-        """ return the given cylindrical `data` in a 3d Cartesian grid
+    def get_cartesian_grid(self, mode: str = 'valid', num: int = None) \
+            -> CartesianGrid:
+        """ return a Cartesian grid for this Cylindrical one 
         
         Args:
-            data (:class:`numpy.ndarray`):
-                The actual data values on the cylindrical grid.
-            grid (str or CartesianGridBase):
-                determines the grid on which the data is returned. This can
-                either be an instance of CartesianGridBase, the string 'valid'
-                or the string 'full'. Using 'valid' only returns points that are
-                fully resolved in the spherical grid using an inscribed
-                sphere. Conversely, 'full' returns all data, using a
-                circumscribed sphere. 
-            ret_grid (bool):
-                Determines whether the respective Rectangular grid should be
-                returned alongside the data.
+            mode (str):
+                Determines how the grid is determined. Setting it to 'valid'
+                only returns points that are fully resolved in the spherical
+                grid, e.g., the sphere is circumscribed. Conversely, 'full'
+                returns all data, so the sphere is inscribed. 
+            num (int):
+                Number of support points along each axis of the returned grid.
                 
         Returns:
-            :class:`numpy.ndarray`: The data values on the Cartesian grid.
-            If `ret_grid=True`, a tuple of these data values and the associated
-            grid are returned. 
+            :class:`pde.grids.cartesian.CartesianGrid`: The requested grid
         """
-        if self.has_hole:
-            raise NotImplementedError
-
-        assert data.shape == self.shape
-        
-        if isinstance(grid, GridBase):
-            # supplied grid is already constructed 
-            if not isinstance(grid, CartesianGridBase):
-                raise TypeError('Supplied grid needs to be of type '
-                                '`CartesianGridBase`')
-            if grid.dim != self.dim:
-                raise ValueError('Dimension mismatch')
-            
+        # Pick the grid instance
+        if mode == 'valid':
+            if self.has_hole:
+                self._logger.warn('The sphere has holes, so not all points are '
+                                  'valid.')
+            bounds = self.radius / np.sqrt(self.dim)
+        elif mode == 'full':
+            bounds = self.radius
         else:
-            # need to build the grid instance
-            if grid == 'valid':
-                bounds = self.radius / np.sqrt(self.dim)
-            elif grid == 'full':
-                bounds = self.radius
-            else:
-                raise ValueError(f'Do not support grid `{grid}`')
+            raise ValueError(f'Unsupported mode `{mode}`')
                 
-            num = round(bounds / self.discretization[0])
-            grid_bounds = [(-bounds, bounds)] * self.dim
-            grid_shape = (2*num,) * self.dim
-            grid = CartesianGrid(grid_bounds, grid_shape)
-        
-        coords = np.meshgrid(*grid.axes_coords, indexing='ij')
-        rs = np.sqrt(sum(c**2 for c in coords))
-        
-        # interpolate over the new coordinates
-        f = interpolate.Rbf(*self.axes_coords, data, function='cubic')
-        data_int = f(rs)
-
-        if ret_grid:
-            return data_int, grid
-        else:
-            return data_int
+        # determine the grid points
+        if num is None:
+            num = 2 * round(bounds / self.discretization[0])
+        grid_bounds = [(-bounds, bounds)] * self.dim
+        return CartesianGrid(grid_bounds, num)
         
 
 
@@ -533,15 +510,16 @@ class PolarGrid(SphericalGridBase):
     The angular symmetry implies that states only depend on the radial
     coordinate :math:`r`, which is discretized uniformly as
     
-    
     .. math::
-        r_i = r_\mathrm{inner} + \left(i + \frac12\right) \Delta r
+        r_i = R_\mathrm{inner} + \left(i + \frac12\right) \Delta r
         \quad \text{for} \quad i = 0, \ldots, N - 1
-        \quad \text{with} \quad \Delta r = \frac{R}{N}
+        \quad \text{with} \quad 
+            \Delta r = \frac{R_\mathrm{outer} - R_\mathrm{inner}}{N}
 
-    where :math:`R` is the radius of the grid and :math:`r_\mathrm{inner}` 
-    corresponds to a possible inner radius, which is zero by default. The radial
-    direction is discretized by :math:`N` support points.    
+    where :math:`R_\mathrm{outer}` is the outer radius of the grid and
+    :math:`R_\mathrm{inner}` corresponds to a possible inner radius, which is
+    zero by default. The radial direction is discretized by :math:`N` support
+    points.    
     """
     
     dim = 2  # dimension of the described space
@@ -561,7 +539,7 @@ class PolarGrid(SphericalGridBase):
         """
         points = np.atleast_1d(points)
         if points.shape[-1] != 1:
-            raise ValueError(f'Dimension mismatch: Points {points} invalid')
+            raise DimensionError(f'Dimension mismatch: Points {points} invalid')
         
         y = points[..., 0]
         x = np.zeros_like(y)
@@ -595,6 +573,7 @@ class PolarGrid(SphericalGridBase):
             
     
     @cached_method()
+    @fill_in_docstring
     def get_operator(self, op: str, bc) -> Callable:
         """ return a discretized operator defined on this grid
         
@@ -602,8 +581,7 @@ class PolarGrid(SphericalGridBase):
             op (str): Identifier for the operator. Some examples are 'laplace',
                 'gradient', or 'divergence'.
             bc (str or list or tuple or dict):
-                Boundary conditions specified in the flexible format accepted by
-                :func:`pde.grids.boundaries.Boundaries.from_data`.  
+                {ARG_BOUNDARIES}
                 
         Returns:
             A function that takes the discretized data as an input and returns
@@ -622,15 +600,16 @@ class SphericalGrid(SphericalGridBase):
     The symmetry implies that states only depend on the radial coordinate 
     :math:`r`, which is discretized as follows:
     
-    
     .. math::
-        r_i = r_\mathrm{inner} + \left(i + \frac12\right) \Delta r
+        r_i = R_\mathrm{inner} + \left(i + \frac12\right) \Delta r
         \quad \text{for} \quad i = 0, \ldots, N - 1
-        \quad \text{with} \quad \Delta r = \frac{R}{N}
+        \quad \text{with} \quad 
+            \Delta r = \frac{R_\mathrm{outer} - R_\mathrm{inner}}{N}
 
-    where :math:`R` is the radius of the grid and :math:`r_\mathrm{inner}` 
-    corresponds to a possible inner radius, which is zero by default. The radial
-    direction is discretized by :math:`N` support points.    
+    where :math:`R_\mathrm{outer}` is the outer radius of the grid and
+    :math:`R_\mathrm{inner}` corresponds to a possible inner radius, which is
+    zero by default. The radial direction is discretized by :math:`N` support
+    points.    
     """
     
     dim = 3  # dimension of the described space
@@ -654,13 +633,14 @@ class SphericalGrid(SphericalGridBase):
         """
         points = np.atleast_1d(points)
         if points.shape[-1] != 1:
-            raise ValueError(f'Dimension mismatch: Points {points} invalid')
+            raise DimensionError(f'Dimension mismatch: Points {points} invalid')
         z = points[..., 0]
         x = np.zeros_like(z)
         return np.stack((x, x, z), axis=-1)    
 
 
     @cached_method()
+    @fill_in_docstring
     def get_operator(self, op, bc: str) -> Callable:
         """ return a discretized operator defined on this grid
         
@@ -668,8 +648,7 @@ class SphericalGrid(SphericalGridBase):
             op (str): Identifier for the operator. Some examples are 'laplace',
                 'gradient', or 'divergence'.
             bc (str or list or tuple or dict):
-                Boundary conditions specified in the flexible format accepted by
-                :func:`pde.grids.boundaries.Boundaries.from_data`.  
+                {ARG_BOUNDARIES}
                 
         Returns:
             A function that takes the discretized data as an input and returns
