@@ -9,11 +9,10 @@ import numpy as np
 import pytest
 
 from .. import trackers
-from ...grids import UnitGrid, CartesianGrid
+from ...grids import UnitGrid
 from ...fields import ScalarField
-from ...pdes import DiffusionPDE
-from ...solvers import ExplicitSolver
-from ...controller import Controller
+from ...pdes import DiffusionPDE, CahnHilliardPDE, AllenCahnPDE
+from ...solvers import ExplicitSolver, Controller
 from ...storage import MemoryStorage
 from ...tools.misc import skipUnlessModule, module_available
         
@@ -167,17 +166,6 @@ def test_small_tracker_dt():
     assert len(storage) == 11
 
 
-
-def test_length_scale_tracker():
-    """ test the LengthScaleTracker """
-    g = CartesianGrid([[0, 2*np.pi]], 64, periodic=True)
-    s = ScalarField.from_expression(g, 'sin(2 * x)')
-
-    tracker = trackers.LengthScaleTracker(0.5)
-    DiffusionPDE().solve(s, t_range=1e-2, dt=1e-3, tracker=tracker)
-    np.testing.assert_allclose(tracker.length_scales, np.pi, rtol=1e-3)
-    
-    
     
 def test_runtime_tracker():
     """ test the RuntimeTracker """
@@ -197,3 +185,22 @@ def test_consistency_tracker():
     with np.errstate(all='ignore'):
         con.run(s, dt=1)
     assert con.info['t_final'] < con.info['t_end']
+
+
+    
+def test_material_conservation_tracker():
+    """ test the MaterialConservationTracker """
+    state = ScalarField.random_uniform(UnitGrid([8, 8]), 0, 1)
+    
+    solver = ExplicitSolver(CahnHilliardPDE())
+    controller = Controller(solver, t_range=1,
+                            tracker=['material_conservation'])
+    controller.run(state, dt=1e-3)
+    assert controller.info['t_final'] >= 1
+    
+    solver = ExplicitSolver(AllenCahnPDE())
+    controller = Controller(solver, t_range=1,
+                            tracker=['material_conservation'])
+    controller.run(state, dt=1e-3)
+    assert controller.info['t_final'] <= 1
+    
\ No newline at end of file
