@@ -11,10 +11,10 @@ The trackers defined in this module are:
    PrintTracker
    PlotTracker
    DataTracker
-   LengthScaleTracker
    SteadyStateTracker
    RuntimeTracker
    ConsistencyTracker
+   MaterialConservationTracker
 
 .. codeauthor:: David Zwicker <david.zwicker@ds.mpg.de>
 """
@@ -30,14 +30,18 @@ import numpy as np
 from .base import TrackerBase, InfoDict, FinishedSimulation, Real
 from .intervals import IntervalData, RealtimeIntervals
 from ..fields.base import FieldBase
+from ..fields import FieldCollection
+from ..visualization.plotting import ScaleData
 from ..tools.parse_duration import parse_duration
 from ..tools.misc import get_progress_bar_class
+from ..tools.docstrings import fill_in_docstring
 
 
 
 class CallbackTracker(TrackerBase):
     """ Tracker that calls a function periodically """
     
+    @fill_in_docstring
     def __init__(self, func: Callable,
                  interval: IntervalData = 1):
         """ 
@@ -49,7 +53,8 @@ class CallbackTracker(TrackerBase):
                 float value indicating the current time. Note that only a view
                 of the state is supplied, implying that a copy needs to be made
                 if the data should be stored.
-            interval: |Arg_tracker_interval|
+            interval:
+                {ARG_TRACKER_INTERVAL}
         """
         super().__init__(interval=interval)
         self._callback = func
@@ -79,12 +84,14 @@ class ProgressTracker(TrackerBase):
             
     name = 'progress'
 
-            
+
+    @fill_in_docstring            
     def __init__(self, interval: IntervalData = None,
                  ndigits: int = 5, leave: bool = True):
         """
         Args:
-            interval: |Arg_tracker_interval|
+            interval:
+                {ARG_TRACKER_INTERVAL}
                 The default value `None` updates the progress bar approximately
                 every (real) second.
             ndigits (int): The number of digits after the decimal point that are
@@ -191,13 +198,16 @@ class PrintTracker(TrackerBase):
     name = 'print'
     
     
+    @fill_in_docstring
     def __init__(self, interval: IntervalData = 1,
                  stream: IO[str] = sys.stdout):
         """
         
         Args:
-            interval: |Arg_tracker_interval|
-            stream: The stream used for printing
+            interval:
+                {ARG_TRACKER_INTERVAL}
+            stream:
+                The stream used for printing
         """
         super().__init__(interval=interval)
         self.stream = stream
@@ -223,15 +233,18 @@ class PlotTracker(TrackerBase):
      
     name = 'plot'
     
+    @fill_in_docstring
     def __init__(self, interval: IntervalData = 1,
                  output_file: Optional[str] = None,
                  output_folder: Optional[str] = None,
                  movie_file: Optional[str] = None,
                  quantities=None,
+                 scale: ScaleData = 'automatic',
                  show: bool = True):
         """
         Args:
-            interval: |Arg_tracker_interval|
+            interval:
+                {ARG_TRACKER_INTERVAL}
             output_file (str, optional):
                 Specifies a single image file, which is updated periodically, so
                 that the progress can be monitored (e.g. on a compute cluster)
@@ -242,7 +255,9 @@ class PlotTracker(TrackerBase):
                 Specifies a filename to which a movie of all the frames is
                 written after the simulation.
             quantities:
-                |Args_plot_quantities|
+                {ARG_PLOT_QUANTITIES}
+            scale (str, float, tuple of float):
+                {ARG_PLOT_SCALE}
             show (bool, optional):
                 Determines whether the plot is shown while the simulation is
                 running. If `False`, the files are created in the background.
@@ -251,6 +266,7 @@ class PlotTracker(TrackerBase):
         self.output_file = output_file
         self.output_folder = output_folder
         self.quantities = quantities
+        self.scale = scale
         self.show = show
         
         if movie_file is not None or output_folder is not None:
@@ -275,7 +291,9 @@ class PlotTracker(TrackerBase):
             float: The first time the tracker needs to handle data
         """
         from ..visualization.plotting import ScalarFieldPlot
-        self.plot = ScalarFieldPlot(field, quantities=self.quantities,
+        self.plot = ScalarFieldPlot(field,
+                                    quantities=self.quantities,
+                                    scale=self.scale,
                                     show=self.show)
          
         return super().initialize(field, info=info)
@@ -327,6 +345,7 @@ class DataTracker(CallbackTracker):
             the callback function. 
     """
     
+    @fill_in_docstring
     def __init__(self, func: Callable,
                  interval: IntervalData = 1):
         """ 
@@ -338,7 +357,8 @@ class DataTracker(CallbackTracker):
                 float value indicating the current time. Note that only a view
                 of the state is supplied, implying that a copy needs to be made
                 if the data should be stored.
-            interval: |Arg_tracker_interval|
+            interval:
+                {ARG_TRACKER_INTERVAL}
         """
         super().__init__(func=func, interval=interval)
         self.times: List[float] = []
@@ -369,93 +389,6 @@ class DataTracker(CallbackTracker):
         df.insert(0, 'time', self.times)
         return df
         
-
-
-class LengthScaleTracker(TrackerBase):
-    """ Tracker that stores length scales measured in simulations
-    
-    Attributes:
-        times (list):
-            The time points at which the length scales are stored
-        length_scales (list):
-            The associated length scales     
-    """
-
-    def __init__(self, interval: IntervalData = 1,
-                 filename: Optional[str] = None,
-                 method: str = 'structure_factor_mean',
-                 source: Union[None, int, Callable] = None,
-                 verbose: bool = False):
-        r"""
-        Args:
-            interval: |Arg_tracker_interval|
-            filename (str, optional): Determines the file to which the data is
-                written.
-            method (str): Method used for determining the length scale. Methods
-                are explain in the function
-                :func:`~pde.analysis.get\_length\_scale`.
-            source (int or callable, optional): Determines how a field is
-                extracted from `fields`. If `None`, `fields` is passed as is,
-                assuming it is already a scalar field. This works for the
-                simple, standard case where only a single ScalarField is
-                treated. Alternatively, `source` can be an integer, indicating
-                which field is extracted from an instance of
-                :class:`~pde.fields.FieldCollection`. Lastly,
-                `source` can be a function that takes `fields` as an argument
-                and returns the desired field.
-            verbose (bool): Determines whether errors in determining the length
-                scales are logged.
-        """
-        super().__init__(interval=interval)
-        self.length_scales: List[float] = []
-        self.times: List[float] = []
-        self.filename = filename
-        self.method = method
-        self.source = source
-        self.verbose = verbose
-        
-        
-    def handle(self, field: FieldBase, t: float):
-        """ handle data supplied to this tracker
-        
-        Args:
-            field (:class:`~pde.fields.FieldBase`):
-                The current state of the simulation
-            t (float): The associated time
-        """
-        # determine length scale
-        from ..visualization.plotting import extract_field
-        from ..analysis import get_length_scale
-                                        
-        scalar_field = extract_field(field, self.source, 0)
-                                        
-        try:
-            length_scale = get_length_scale(scalar_field,  # type: ignore
-                                            method=self.method)
-        except Exception:
-            if self.verbose:
-                self._logger.exception('Could not determine length scale')
-            length_scale = np.nan
-            
-        # store data
-        self.times.append(t)
-        self.length_scales.append(length_scale)
-        
-        
-    def finalize(self, info: InfoDict = None) -> None:
-        """ finalize the tracker, supplying additional information
-
-        Args:
-            info (dict):
-                Extra information from the simulation        
-        """
-        super().finalize(info)
-        if self.filename:
-            import json
-            data = {'times': self.times, 'length_scales': self.length_scales}
-            with open(self.filename, 'w') as fp:
-                json.dump(data, fp)
-            
             
             
 class SteadyStateTracker(TrackerBase):
@@ -468,12 +401,14 @@ class SteadyStateTracker(TrackerBase):
     name = 'steady_state'
 
 
+    @fill_in_docstring
     def __init__(self, interval: IntervalData = None,
                  atol: float = 1e-8,
                  rtol: float = 1e-5):
         """
         Args:
-            interval: |Arg_tracker_interval|
+            interval:
+                {ARG_TRACKER_INTERVAL}
                 The default value `None` checks for the steady state
                 approximately every (real) second.
             atol (float): Absolute tolerance that must be reached to abort the
@@ -507,6 +442,7 @@ class RuntimeTracker(TrackerBase):
     """ Tracker that interrupts the simulation once a duration has passed """
 
 
+    @fill_in_docstring
     def __init__(self, max_runtime: Union[Real, str],
                  interval: IntervalData = 1):  
         """
@@ -517,7 +453,8 @@ class RuntimeTracker(TrackerBase):
                 given as a number (interpreted as seconds) or as a string, which
                 is then parsed using the function
                 :func:`~pde.tools.parse_duration.parse_duration`.
-            interval: |Arg_tracker_interval|
+            interval:
+                {ARG_TRACKER_INTERVAL}
         """
         super().__init__(interval=interval)
         
@@ -556,12 +493,15 @@ class ConsistencyTracker(TrackerBase):
 
     name = 'consistency'
         
-        
+    
+    @fill_in_docstring
     def __init__(self, interval: IntervalData = None):
         """
         Args:
-            interval: |Arg_tracker_interval| The default value `None` checks for
-                consistency approximately every (real) second.
+            interval: 
+                {ARG_TRACKER_INTERVAL}
+                The default value `None` checks for consistency approximately
+                every (real) second.
         """ 
         if interval is None:
             interval = RealtimeIntervals(duration=1)
@@ -576,3 +516,67 @@ class ConsistencyTracker(TrackerBase):
         self._last = field.data.copy()  # store data from last timestep
             
 
+
+class MaterialConservationTracker(TrackerBase):
+    """ Ensure that the amount of material is conserved """
+
+    name = 'material_conservation'
+
+
+    @fill_in_docstring
+    def __init__(self, interval: IntervalData = 1,
+                 atol: float = 1e-4,
+                 rtol: float = 1e-4):
+        """
+        Args:
+            interval:
+                {ARG_TRACKER_INTERVAL}
+            atol (float):
+                Absolute tolerance for amount deviations
+            rtol (float):
+                Relative tolerance for amount deviations
+        """
+        super().__init__(interval=interval)
+        self.atol = atol 
+        self.rtol = rtol
+        
+        
+    def initialize(self, field: FieldBase, info: InfoDict = None) -> float:
+        """ 
+        Args:
+            field (:class:`~pde.fields.base.FieldBase`):
+                An example of the data that will be analyzed by the tracker
+            info (dict):
+                Extra information from the simulation        
+                
+        Returns:
+            float: The first time the tracker needs to handle data
+        """
+        if isinstance(field, FieldCollection):
+            self._reference = np.array([f.magnitude for f in field])
+        else:
+            self._reference = field.magnitude  # type: ignore
+            
+        return super().initialize(field, info)
+        
+        
+    def handle(self, field: FieldBase, t: float) -> None:
+        """ handle the data of `field` for a give `time` """
+        if isinstance(field, FieldCollection):
+            mags = np.array([f.magnitude for f in field])
+        else:
+            mags = field.magnitude  # type: ignore
+            
+        c = np.isclose(mags, self._reference, rtol=self.rtol, atol=self.atol)
+        if not np.all(c):
+            if isinstance(field, FieldCollection):
+                msg = f'Material of field {np.flatnonzero(~c)} is not conserved'
+            else:
+                msg = f'Material is not conserved'
+            raise StopIteration(msg)
+            
+            
+            
+__all__ = ['CallbackTracker', 'ProgressTracker', 'PrintTracker', 'PlotTracker',
+           'DataTracker', 'SteadyStateTracker', 'RuntimeTracker',
+           'ConsistencyTracker', 'MaterialConservationTracker']
