@@ -14,7 +14,7 @@ Functions and classes for plotting simulation data
 
 import logging
 import warnings
-from typing import Union, Callable, Optional, Any, List  # @UnusedImport
+from typing import (Union, Callable, Optional, Any, Dict, List, Tuple)
 
 import numpy as np
 
@@ -22,6 +22,10 @@ from ..fields import FieldCollection
 from ..fields.base import FieldBase, DataFieldBase
 from ..storage.base import StorageBase
 from ..tools.misc import display_progress
+from ..tools.docstrings import fill_in_docstring
+
+
+ScaleData = Union[str, float, Tuple[float, float]]
 
 
 
@@ -140,21 +144,31 @@ def extract_field(fields: FieldBase,
 class ScalarFieldPlot():
     """ class managing compound plots of scalar fields """
     
-    def __init__(self, fields: FieldBase, quantities=None, show: bool = True):
+    @fill_in_docstring
+    def __init__(self, fields: FieldBase,
+                 quantities=None,
+                 scale: ScaleData = 'automatic',
+                 show: bool = True):
         """
         Args:
-            fields: Collection of fields
-            quantities: |Args_plot_quantities|
-            show (bool): Flag determining whether to show a plot. If `False`,
-                the plot is kept in the background, which can be useful if
-                it only needs to be written to a file
+            fields:
+                Collection of fields
+            quantities:
+                {ARG_PLOT_QUANTITIES}
+            scale (str, float, tuple of float):
+                {ARG_PLOT_SCALE}
+            show (bool):
+                Flag determining whether to show a plot. If `False`, the plot is
+                kept in the background, which can be useful if it only needs to
+                be written to a file
         """
         import matplotlib.pyplot as plt
         import matplotlib.cm as cm
         
         self.grid = fields.grid
         example_image = fields.get_image_data()
-        self.quantities = self._prepare_quantities(fields, quantities)
+        self.quantities = self._prepare_quantities(fields, quantities,
+                                                   scale=scale)
         self.show = show
              
         num_rows = len(self.quantities)
@@ -174,16 +188,25 @@ class ScalarFieldPlot():
             img_row = []
             for j, panel in enumerate(panel_row):
                 # determine scale of the panel
-                scale = panel.get('scale', 1)
-                try:
-                    vmin, vmax = scale
-                except TypeError:
-                    vmin, vmax = 0, scale
+                panel_scale = panel.get('scale', scale)
+                if panel_scale == 'automatic':
+                    vmin, vmax = None, None
+                elif panel_scale == 'unity':
+                    vmin, vmax = 0, 1
+                elif panel_scale == 'symmetric':
+                    vmin, vmax = -1, 1
+                else:
+                    try:
+                        vmin, vmax = panel_scale
+                    except TypeError:
+                        vmin, vmax = 0, panel_scale
                     
                 # determine colormap
                 cmap = panel.get('cmap')
                 if cmap is None:
-                    if np.isclose(-vmin, vmax):
+                    if vmin is None or vmax is None:
+                        cmap = cm.PiYG
+                    elif np.isclose(-vmin, vmax):
                         cmap = cm.coolwarm
                     elif np.isclose(vmin, 0):
                         cmap = cm.gray
@@ -199,7 +222,9 @@ class ScalarFieldPlot():
                 ax.set_title(panel.get('title', ''))
                 ax.set_xlabel(example_image['label_x'])
                 ax.set_ylabel(example_image['label_y'])
-#                 ax.axison = False
+                
+                # store the default value alongside
+                img._default_vmin_vmax = vmin, vmax
                 img_row.append(img)
             self.images.append(img_row)
             
@@ -207,20 +232,27 @@ class ScalarFieldPlot():
         
     
     @classmethod
-    def from_storage(cls, storage: StorageBase, quantities=None):
+    @fill_in_docstring
+    def from_storage(cls, storage: StorageBase,
+                     quantities=None,
+                     scale: ScaleData = 'automatic') \
+            -> "ScalarFieldPlot":
         """ create ScalarFieldPlot from storage
         
         Args:
             storage (:class:`~pde.storage.base.StorageBase`):
                 Instance of the storage class that contains the data
-            quantities: |Args_plot_quantities|
+            quantities:
+                {ARG_PLOT_QUANTITIES}
+            scale (str, float, tuple of float):
+                {ARG_PLOT_SCALE}
             
         Returns:
             :class:`~pde.visualization.plotting.ScalarFieldPlot`
         """
         fields = storage.get_field(0)
-        quantities = cls._prepare_quantities(fields, quantities, 
-                                             default_scale='automatic')
+        quantities = cls._prepare_quantities(fields, quantities=quantities,
+                                             scale=scale)
         
         # resolve automatic scaling
         for quantity_row in quantities:
@@ -235,21 +267,36 @@ class ScalarFieldPlot():
                     quantity['scale'] = (vmin, vmax)
             
         # actually setup 
-        return cls(fields, quantities=quantities)
+        return cls(fields, quantities)  # lgtm [py/call-to-non-callable]
         
         
     @staticmethod
-    def _prepare_quantities(fields: FieldBase, quantities, default_scale=None):
-        """ internal method to prepare quantities """
+    @fill_in_docstring
+    def _prepare_quantities(fields: FieldBase,
+                            quantities,
+                            scale: ScaleData = 'automatic') \
+            -> List[List[Dict[str, Any]]]:
+        """ internal method to prepare quantities
+        
+        Args:
+            fields (:class:`~pde.fields.FieldBase`):
+                The field containing the data to show
+            quantities (dict):
+                {ARG_PLOT_QUANTITIES}
+            scale (str, float, tuple of float):
+                {ARG_PLOT_SCALE}
+        
+        Returns:
+            list of list of dict: a 2d arrangements of panels that define what
+                quantities are shown.
+        """
         if quantities is None:
             # show each component by default
             if isinstance(fields, FieldCollection):
                 quantities = []
                 for i, field in enumerate(fields):
                     title = field.label if field.label else f'Field {i + 1}'
-                    quantity = {'title': title, 'source': i}
-                    if default_scale:
-                        quantity['scale'] = default_scale
+                    quantity = {'title': title, 'source': i, 'scale': scale}
                     quantities.append(quantity)
             else:
                 quantities = [{'title': 'Concentration', 'source': None}]
@@ -259,7 +306,7 @@ class ScalarFieldPlot():
             quantities = [[quantities]]
         elif isinstance(quantities[0], dict):
             quantities = [quantities]
-        return quantities
+        return quantities  # type: ignore
         
         
     def __del__(self):
@@ -272,6 +319,20 @@ class ScalarFieldPlot():
                 plt.close(self.fig)
         
             
+    def savefig(self, path: str, **kwargs):
+        """ save plot to file 
+        
+        Args:
+            path (str):
+                The path to the file where the image is written. The file
+                extension determines the image format
+            **kwargs:
+                Additional arguments are forwarded to 
+                :meth:`matplotlib.figure.Figure.savefig`.
+        """
+        self.fig.savefig(path, **kwargs)
+        
+            
     def show_data(self, fields: FieldBase, title: Optional[str] = None) -> None:
         """ show the given fields in the current view
         
@@ -289,9 +350,19 @@ class ScalarFieldPlot():
         
         for i, panel_row in enumerate(self.quantities):
             for j, panel in enumerate(panel_row):
+                # obtain image data
                 field = extract_field(fields, panel.get('source'))
-                img = field.get_image_data()
-                self.images[i][j].set_data(img['data'])
+                img_data = field.get_image_data()
+                
+                # set the data in the correct panel
+                img = self.images[i][j]
+                img.set_data(img_data['data'])
+                vmin, vmax = img._default_vmin_vmax
+                if vmin is None:
+                    vmin = img_data['data'].min()
+                if vmax is None:
+                    vmax = img_data['data'].max()
+                img.set_clim(vmin, vmax)
                 
         # add a small pause to allow the GUI to run it's event loop
         if self.show:
@@ -315,9 +386,11 @@ class ScalarFieldPlot():
         """
         from ..visualization.movies import Movie
 
-        data_iter = storage.items()
+        # create the iterator over the data
         if progress:
-            data_iter = display_progress(data_iter, total=len(storage))
+            data_iter = display_progress(storage.items(), total=len(storage))
+        else:
+            data_iter = storage.items()
             
         with Movie(filename=filename, verbose=False) as movie:
             # iterate over all time steps
@@ -327,6 +400,7 @@ class ScalarFieldPlot():
 
 
 
+@fill_in_docstring
 def plot_magnitudes(storage: StorageBase,
                     filename: str = None,
                     quantities=None,
@@ -344,7 +418,8 @@ def plot_magnitudes(storage: StorageBase,
             the simulation data that will be plotted
         filename (str):
             If given, the resulting image is written to this file.
-        quantities: |Args_plot_quantities|
+        quantities:
+            {ARG_PLOT_QUANTITIES}
         ax:
             Matplotlib figure axes to be used for plotting. If `None`, a new
             figure is created
@@ -457,7 +532,7 @@ def plot_kymograph(storage: StorageBase,
     """
     import matplotlib.pyplot as plt
 
-    close_figure = (filename != '' and ax is None)
+    close_figure = (bool(filename) and ax is None)
     if ax is None:
         # create new figure
         ax = plt.figure().gca()
